#
# db2_profile::database::db_standby
#
# @summary When the property `db2_profile::database::primary` is set, and the current node is not the designated primary server, this class will be applied.
# When the named database does not yet run on the server, Puppet will use the backup at the `backup_path` to create an intial running database and set it in standby mode.
#
# @param [Hash] list
#    A hash of `db2_database` entries.
#
# @param [String[1]] instance
#    The instance to use for the database.
#
# @param [String[1]] database
#    The database name to use for the primary or standby database.
#    this needs to be the same on the primary and standby nodes.
#
# @param [Stdlib::Absolutepath] backup_path
#    The path where DB2 can find the backup of this database.
#    Before applying this class, you'll have to use a `before_db_standby` class to copy the backup from the primary DB2 node to the location specified by this parameter.
#
# @param [Variant[Boolean, Enum['on_failure']]] logoutput
#    If you want to see the output of the `exec` resources in the type, you can set this value to `true`.
#    The default is `on_failure`.
#    Here is an example on how to use this:
#        class { '::db2_install::...':
#          ...
#          logoutput => true,
#          ...
#        }
#
#
# See the file "LICENSE" for the full license governing this code.
#
class db2_profile::database::db_standby (
  Stdlib::Absolutepath $backup_path,
  String[1]            $database,
  String[1]            $instance,
  Hash                 $list,
  Variant[Boolean,Enum['on_failure']]
  $logoutput = lookup( { name => 'logoutput', default_value => 'on_failure' })
) {
  easy_type::debug_evaluation() # Show local variable on extended debug

  if $facts['db2_databases'].dig("${instance}/${database}") == undef {
    echo { "Ensure DB2 standby database based on backup in '${$backup_path}'":
      withpath => false,
    }

    exec { "restore backup for ${database}":
      command     => "/home/${instance}/sqllib/bin/db2 restore database ${database} from ${backup_path}",
      cwd         => $backup_path,
      user        => $instance,
      environment => "DB2INSTANCE=${instance}",
      logoutput   => $logoutput,
    }

    exec { "set database '${database}' as standby":
      command     => "/home/${instance}/sqllib/bin/db2 start hadr on database ${database} as standby",
      user        => $instance,
      environment => "DB2INSTANCE=${instance}",
      logoutput   => $logoutput,
    }

    $require = Exec["restore backup for ${database}"]
    $before  = Exec["set database '${database}' as standby"]
  } else {
    $require = undef
    $before  = undef
  }

  $list.each |$name, $properties| {
    db2_database { $name:
      allow_restart => true,
      *             => $properties,
      require       => $require,
      before        => $before,
    }
  }
}
